/**
 *  Copyright (c) 2010 A.G.Docherty. All Rights Reserved.
 */

package uk.co.keang.sudoku;

import java.util.ArrayList;
import java.util.List;

import uk.co.keang.sudoku.event.SolutionEvent;

/**
 * @author A.G.Docherty
 *
 */
public class SequenceBuffer implements SolverListener
{
private final List<SolutionEvent> buf = new ArrayList<SolutionEvent>();
private Puzzle initialGrid;

@Override
public void removedPossibleValues(SolutionEvent event)
	{
	buf.add(event);
	}

public List<SolutionEvent> getCoalescedEvents()
    {
    List<SolutionEvent> coalesced = new ArrayList<SolutionEvent>();
    
    if ( buf.isEmpty() )
    	return coalesced;
    
    int index = -1;
    for ( int i = 0; i < buf.size(); i++ )
	    {
	    SolutionEvent e = buf.get(i);
	    
	    if ( i > 0 && buf.get(i-1).canCoalesce(e) )
	    	{
	    	coalesced.get(index).coalesce(e);
	    	}
    	else
    		{
    		coalesced.add(e.clone());
    		index++;
    		}
	    }
    
    return coalesced;
    }

@Override
public String toString()
    {
    List<SolutionEvent> b = getCoalescedEvents();
    StringBuffer sb = new StringBuffer();
    
    for ( SolutionEvent e : b )
	    {
	    sb.append(e.toString());
	    sb.append("\n");
	    }

    return sb.toString();
    }

/**
 * Sets the start grid
 * 
 * @param grid
 */
public void setStartGrid(Puzzle grid)
	{
	initialGrid = grid.clone();
	}

/** 
 * Gets the start grid
 * 
 * @return the grid
 */
public Puzzle getStartGrid()
	{
	return initialGrid;
	}

public void clear()
	{
	buf.clear();
	}
}
