/**
 *  Copyright (c) 2010 A.G.Docherty. All Rights Reserved.
 */

package uk.co.keang.sudoku;

import java.util.NoSuchElementException;
import java.util.Scanner;

import uk.co.keang.sudoku.Puzzle.Cell;
import uk.co.keang.sudoku.event.IllegalGridException;

/**
 * A command line interface Sudoku solver
 * 
 * @author A.G.Docherty
 *
 */
public class SudokuCLI
{
private final SudokuSolver sudoku;
private final Scanner scanner;

public SudokuCLI(String[] args)
    {
    sudoku = new SudokuSolver(args);
    scanner = new Scanner(System.in);
    
    runSolver();
    sudoku.print();
    }

private void runSolver()
	{
	while (true)
		{
		String inp = getInput().toUpperCase();
	
		if ( inp.startsWith("E") || inp.startsWith("Q") )
			return;
		else if ( inp.startsWith("L") )
			System.out.println(sudoku.getSequenceBuffer().toString());
		else if ( inp.startsWith("S") )
			sudoku.print();
		else if ( inp.startsWith("D") )
			sudoku.printDetail();
		else if ( inp.startsWith("C") )
			{
			try
	            {
	            sudoku.compute();
	            }
            catch ( IllegalGridException exc )
	            {
				System.out.println("ERROR: the sudoku contains illegal values: "+exc.getMessage());
	            }
			}
		else if ( inp.contains(",") )
			{
			// get the x, y and num
			String[] args = inp.split(",");
			
			if ( args.length == 3 )
				{
				int x = Integer.parseInt(args[0]);
				int y = Integer.parseInt(args[1]);
				int num = Integer.parseInt(args[2]);
				
				if ( !sudoku.isInBounds(x) )
					continue;
				
				if ( !sudoku.isInBounds(y) )
					continue;
				
				if ( !sudoku.isInBounds(num) )
					continue;
				
				Puzzle puzzle = sudoku.getPuzzle();
				Cell orig = puzzle.set(x-1, y-1, num);
				
				if ( !puzzle.isLegal() )
					{
					puzzle.set(x-1, y-1, orig);
					System.out.println("Setting square at "+x+", "+y+" to "+num+" is illegal");
					}
				}
			}
		}
	}

private String getInput()
	{
	System.out.println();
	System.out.print("> ");
	
	String line;
    try
	    {
	    line = scanner.nextLine();
	    }
    catch ( NoSuchElementException exc )
	    {
	    // this can occur on hard closing the application
	    System.out.println("Shutting Down");
	    return "Q";
	    }
    
	return line;
	}


/**
 * @param args
 */
@SuppressWarnings("unused")
public static void main(String[] args)
	{
	if ( args.length != 81 || 
		(args.length == 1 && args[0].length() != 81) )
		{
		System.out.println("There must be 81 numbers/spaces, you have entered "+args.length);
		
		args = new String[]{"    358   1 7  9  6.7       2 8   5  3  6  8  4   2 6       4 8  2  9 1   138    "};
//		args = new String[]{"   2 1  4  8 9 3   4 6   9 51  7 2   6   3 7 4 2    31 3   2 4  8475 1     3 6 2 "};
//		return;
		}

	new SudokuCLI(args);
	}


}
