/**
 *  Copyright (c) 2010 A.G.Docherty. All Rights Reserved.
 */

package uk.co.keang.sudoku.event;

import java.awt.Point;
import java.util.Arrays;
import java.util.List;

import uk.co.keang.sudoku.Puzzle;
import uk.co.keang.sudoku.Puzzle.Cell;
import uk.co.keang.sudoku.solver.AbstractSolver;

/**
 * A solution event
 * 
 * @author A.G.Docherty
 *
 */
public class AssociatedPairsEvent extends SolutionEvent implements Cloneable
{
private static final long serialVersionUID = -8337084924318825911L;

/**
 * A solver event for a changed cell
 * 
 * @param src  the solver that generated the event
 * @param msg  a descriptive message from the solver
 * @param grid  the grid after the proposed changes are applied
 * @param loc  the location (0 based) of the cell that has been modified
 * @param removed  the values that been removed from this cells possible values list
 * @param from  a list of locations (0 based) of the cells used to determine which values can be removed 
 * @throws IllegalGridException 
 */
public AssociatedPairsEvent(AbstractSolver src, String msg, Puzzle grid, Point loc, Cell removed, List<Point> from, Grouping group) throws IllegalGridException
	{
	super(src, msg, grid, Arrays.asList(loc), removed, from, group);
	setDisplayOrder(DisplayOrder.REFERENCE_FIRST);
	}

/**
 * Returns true if the source and location are the same and if they both have change sets
 * 
 * @param e
 * @return
 */
@Override
public boolean canCoalesce(SolutionEvent e)
	{
	if ( e == null )
		return false;
	
	if ( source != e.getSource() )
		return false;
	
	if ( getGrouping() != e.getGrouping() )
		return false;

	if ( !getReferencedLocations().equals(e.getReferencedLocations()) )
		return false;

	if ( !getRemovedValues().equals(e.getRemovedValues()) )
		return false;
	
	return true;
	}

}
